/*
 * BearLibFOV.h
 *
 *  Created on: Jun 6, 2013
 *      Author: cfyz
 */

#ifndef BEARLIBFOV_H
#define BEARLIBFOV_H

#ifdef _WIN32
#  ifdef BEARLIBFOV_LIBRARY
#    define FOV_API __declspec(dllexport)
#  else
#    define FOV_API __declspec(dllimport)
#  endif
#else
#  define FOV_API
#endif

#ifdef __cplusplus
extern "C" {
#endif

/**
 * FOV callback
 * @param map_id BearLibMap object FOV being calculated on
 * @param x visible cell coordinates
 * @param y visible cell coordinates
 * @param opaque pointer, supplied to fov_calc_cb function
 */
typedef void (*fov_callback)(int map_id, int x, int y, void* opaque);

/**
 * FOV calculation
 * @param map_id BearLibMap object to calculate FOV on
 * @param opacity_layer map layer with opacity information
 * @param x player position
 * @param y player position
 * @param radius player sight radius
 * @param callback function to be called with visible cells coordinates
 * @param opaque user-supplied pointer passed to callback function
 * @return 0 on success, -1 otherwise
 */
FOV_API int fov_calc_cb(int map_id, int opacity_layer, int x, int y, int radius, fov_callback callback, void* opaque);

/**
 * FOV calculation, simplified version without callback
 * @param map_id BearLibMap object to calculate FOV on
 * @param opacity_layer map layer with opacity information
 * @param visibility_layer map layer where visible cells will be marked
 * @param x player position
 * @param y player position
 * @param radius player sight radius
 * @return 0 on success, -1 otherwise
 */
FOV_API int fov_calc(int map_id, int opacity_layer, int visibility_layer, int x, int y, int radius);

#undef FOV_API

#ifdef __cplusplus
} // extern "C"
#endif

#endif // BEARLIBFOV_H
