/*
 * Utility.hpp
 *
 *  Created on: Jun 7, 2013
 *      Author: cfyz
 */

#ifndef UTILITY_HPP_
#define UTILITY_HPP_

#include <string>
#include <list>

namespace BearLib
{
	template<typename char_t> std::basic_string<char_t> trim(const std::basic_string<char_t>& what)
	{
		if ( what.empty() ) return what;

		const char_t whitespace[3] = { char_t(' '), char_t('\t'), 0 };
		size_t left = what.find_first_not_of(whitespace);
		size_t right = what.find_last_not_of(whitespace);

		if ( left == std::basic_string<char_t>::npos )
		{
			return std::basic_string<char_t>();
		}
		else
		{
			return what.substr(left, right-left+1);
		}
	}

	template<typename char_t, typename func> void split(const std::basic_string<char_t>& what, char_t on, func f)
	{
		size_t left = 0, right = 0;

		auto insert_fn = [&what, &f](size_t left, size_t right) -> void
		{
			if ( left < right ) f(trim(what.substr(left, right-left)));
		};

		for ( ; right<what.length(); right++ )
		{
			if ( what[right] == on )
			{
				insert_fn(left, right);
				left = right+1;
			}
		}

		insert_fn(left, right);
	}

	template<typename char_t> std::list<std::basic_string<char_t>> split(const std::basic_string<char_t>& what, char_t on)
	{
		std::list<std::basic_string<char_t>> result;
		split(what, on, [&result](const std::basic_string<char_t>& s) { result.emplace_back(s); });
		return result;
	}
}

#endif /* UTILITY_HPP_ */
