/*
 * BasicOutput.cpp
 *
 *  Created on: Jun 23, 2013
 *      Author: Cfyz
 */

#include "BearLibTerminal.h"
#include "Demos.hpp"

static void DrawOverlayRectangle(int x, int y, int w, int h, color_t border, color_t background);

void BasicOutput()
{
	terminal_options("title=BearLibTerminal demo: output facilities");
	terminal_options("font.name=default; font.supplement=media\\supplement-runic.png");

	terminal_blending(BLENDING_ADDITIVE);
	terminal_color(COLOR_WHITE);

	terminal_clear();

	// Simple output with postformatting
	terminal_printf(1, 1, "[color=gray]1. Most basic:[/color] ABC abc");

	// Wide color range
	int n = terminal_printf(1, 3, "[color=gray]2. Colors: ");
	const char long_word[] = "Antidisestablishmentarianism";
	const size_t long_word_length = sizeof(long_word)-1;
	for ( size_t i = 0; i < long_word_length; i++ )
	{
		float factor = float(i)/long_word_length;
		int red = (1.0f - factor) * 255;
		int green = factor * 255;
		terminal_color(color_from_argb(255, red, green, 0));
		terminal_put(n+1+i, 3, long_word[i]);
	}
	terminal_color(color_from_name("white"));

	// Tile backgrounds
	terminal_printf(1, 5, "[color=gray]3. Backgrounds: [color=black][bkcolor=gray] grey [/bkcolor] [bkcolor=red] red ");

	// Simple unicode output
	terminal_printf(1, 7, "[color=gray]4. Unicode:[/color] Latin Русский Ελληνικά");

	// Composite (overlay) tiles
	n = terminal_printf(1, 9, "[color=gray]5. Tile overlay: ");
	terminal_printf(n+1, 9, "a+  = a, a+  = a, a vs. [color=red]ä");
	terminal_printf(n+1, 9, "[color=red]  /   /    ▔   ▔        [/color]a");

	// Unicode Box Drawing characters
	terminal_printf(1, 11, "[color=gray]6. Box drawing:");
	const char* box_lines[] =
	{
		"       ┌──┬───┐      ╔═════╗ ",
		" ┌───┬─┘  │   ├──┐   ╟──┐  ║ ",
		" └─┐ └─┐  ├───┘  │   ║  └──╢ ",
		"   └───┴──┴──────┘   ╚═════╝ "
	};
	for ( size_t i = 0; i < sizeof(box_lines)/sizeof(box_lines[i]); i++ )
	{
		terminal_printf(1, 12+i, "%s", box_lines[i]);
	}

	// Box drawing + overlay application
	terminal_printf(1, 17, "[color=gray]7. Box drawing + overlay:");
	terminal_color(0xFF006000);
	for ( int y = 0; y < 6; y++ )
	{
		for ( int x = 0; x < 19; x++ )
		{
			terminal_put(1+x, 18+y, 'A'+(x+y)%26);
		}
	}
	DrawOverlayRectangle(3, 19, 15, 4, 0xFFFFFFFF, 0xA0909090);
	terminal_printf(4, 20, "[color=white]Itym:\nHeavy draggon");

	terminal_printf(40, 17, "[color=gray]8. Font supplement:");
	terminal_printf(40, 19, "fire ([color=red][uE001][/color]) - water ([color=lighter blue][uE004][/color]) - earth ([color=dark green][uE00F][/color])");
	terminal_printf(40, 20, "\uE001\uE002\uE003, \uE004\uE005\uE006\uE007\uE008! \uE009\uE00A\uE00B \uE00C \uE00D\uE00E\uE00F\uE010.");

	terminal_refresh();

	int key = 0;
	do
	{
		key = terminal_get();
	}
	while ( key != VK_CLOSE && key != VK_ESCAPE );
}

void DrawOverlayRectangle(int x, int y, int w, int h, color_t border, color_t background)
{
	const wchar_t
		kLowerRightQuad = 0x2597,	kLowerHalf = 0x2584,	kLowerLeftQuad = 0x2596,
		kRightHalf = 0x2590,		kFullBlock = 0x2588,	kLeftHalf = 0x258C,
		kUpperRightQuad = 0x259D,	kUpperHalf = 0x2580,	kUpperLeftQuad = 0x2598;

	const wchar_t
		kUpperLeftCorner = 0x250C,	kHorisontal = 0x2500,	kUpperRightCorner = 0x2510,
		kVertical = 0x2502,			/* none */				/* kVertical */
		kLowerLeftCorner = 0x2514,	/* kHorisontal */		kLowerRightCorner = 0x2518;

	// First, background
	terminal_color(background);
	for ( int i = x+1; i < (x+w)-1; i++ )
	{
		for ( int j = y+1; j < (y+h)-1; j++ )
		{
			terminal_put(i, j, kFullBlock);
		}

		terminal_put(i, y, kLowerHalf);
		terminal_put(i, y+h-1, kUpperHalf);
	}
	for ( int j = y+1; j < (y+h)-1; j++ )
	{
		terminal_put(x, j, kRightHalf);
		terminal_put(x+w-1, j, kLeftHalf);
	}
	terminal_put(x, y, kLowerRightQuad);
	terminal_put(x+w-1, y, kLowerLeftQuad);
	terminal_put(x, y+h-1, kUpperRightQuad);
	terminal_put(x+w-1, y+h-1, kUpperLeftQuad);

	// Then, border
	terminal_color(border);
	for ( int i = x+1; i < (x+w)-1; i++ )
	{
		terminal_put(i, y, kHorisontal);
		terminal_put(i, y+h-1, kHorisontal);
	}
	for ( int j = y+1; j < (y+h)-1; j++ )
	{
		terminal_put(x, j, kVertical);
		terminal_put(x+w-1, j, kVertical);
	}
	terminal_put(x, y, kUpperLeftCorner);
	terminal_put(x+w-1, y, kUpperRightCorner);
	terminal_put(x, y+h-1, kLowerLeftCorner);
	terminal_put(x+w-1, y+h-1, kLowerRightCorner);
}
