/*
 * Codepages.cpp
 *
 *  Created on: Jun 23, 2013
 *      Author: Cfyz
 */

// ...

#include "BearLibTerminal.h"
#include "Demos.hpp"

void Codepages()
{
	terminal_options("title=BearLibTerminal demo: codepages");

	struct codepage
	{
		int code;
		const char* desc;
	}
	codepages[] =
	{
		{ CODEPAGE_DEFAULT, "Unicode" },
		{ 1250, "Windows Latin (central European)" },
		{ 1251, "Windows Cyrilic" },
		{ 1252, "Windows Latin (western European)" },
		{ 1253, "Windows Greek" },
		{ 1257, "Windows Latin (baltic)" },
		{ 437, "DOS Latin (US)" },
		{ 737, "DOS Greek" },
		{ 852, "DOS Latin (central Europeran)" },
		{ 858, "DOS Latin (western Europeran)" },
		{ 866, "DOS Cyrillic" }
	};

	const int N_codepages = sizeof(codepages)/sizeof(codepages[0]);

	int current_codepage = 0;

	color_t menu_basic = 0xFFFFFFFF;
	color_t menu_selected = 0xFFFF8000;
	color_t menu_descripton = 0xFFA0A0A0;
	color_t range_numeric = 0xFF008000;
	color_t range_text = menu_basic;

	const auto reinitialize = [&](int codepage)
	{
		terminal_close();
		terminal_open(codepage);
		terminal_options("size=80x25; font.name=default; title=BearLibTerminal demo: codepages");
	};

	bool unconditional_close = false;

	do
	{
		terminal_clear();

		for ( int i = 0; i < N_codepages; i++ )
		{
			terminal_color(i == current_codepage? menu_selected: menu_basic);
			terminal_printf(1, 1+i, "%d", codepages[i].code);
			terminal_color(menu_descripton);
			terminal_printf(1+8, 1+i, "%s", codepages[i].desc);
		}

		terminal_color(0xFF808000);
		terminal_printf(1, N_codepages+2, "Tip: use up/down arrow keys");

		terminal_color(range_numeric);
		for ( int column = 0; column < 16; column++ )
		{
			terminal_printf(80-33+1+column*2, 1, "%X", column);
		}

		for ( int i = 0; i < 256; )
		{
			int row = i/16;
			terminal_color(range_numeric);
			terminal_printf(80-33-4, 2+row, "%04X", i);

			terminal_color(range_text);
			for ( int column = 0; column < 16; column++, i++ )
			{
				terminal_put(80-33+1+column*2, 2+row, i);
			}
		}
		terminal_refresh();

		int code = terminal_get();
		if ( code == VK_CLOSE || code == VK_ESCAPE )
		{
			unconditional_close = code == VK_CLOSE;
			break;
		}
		else if ( code == VK_UP )
		{
			if ( current_codepage > 0 )
			{
				current_codepage -= 1;
				reinitialize(codepages[current_codepage].code);
			}
		}
		else if ( code == VK_DOWN )
		{
			if ( current_codepage < N_codepages-1 )
			{
				current_codepage += 1;
				reinitialize(codepages[current_codepage].code);
			}
		}
	}
	while ( true );

	// Must not reopen terminal in case of 'close window' command, as it will be lost
	if ( !unconditional_close ) reinitialize(CODEPAGE_DEFAULT);
}
