/*
 * Flare.cpp
 *
 *  Created on: Jun 23, 2013
 *      Author: Cfyz
 */

// ...

#include "BearLibTerminal.h"
#include "Demos.hpp"
#include <Windows.h>
#include <cmath>

void Flare()
{
	terminal_options("title=BearLibTerminal demo: flare; input.timeout=%d", 40);

	const int W = 80;
	const int H = 25;

	int tiles[W][H];

	int double_lines[16] = { 0, 0, 0, 0x255A, 0, 0x2551, 0x2554, 0x2560, 0, 0x255D, 0x2550, 0x2569, 0x2557, 0x2563, 0x2566, 0x256C };

	// Some rectangles
	memset(tiles, 0, sizeof(tiles));
	int x = 5, y = 1, w = 70, h = 22;

	for ( int d = 1; w >= 3 && h >= 3; d=(d+1)%4 )
	{
		for ( int j = x; j <= x+w; j++ ) { tiles[j][y] = 1; tiles[j][y+h] = 1; }
		for ( int j = y; j <= y+h; j++ ) { tiles[x][j] = 1; tiles[x+w][j] = 1; }

		int N = (d%2)? w: h;
		int n = rand()%(1+int(N * 0.25f))+3;

		if ( d == 0 ) { y += n; h -= n; }
		else if ( d == 1 ) { w -= n; }
		else if ( d == 2 ) { h -= n; }
		else { x += n; w -= n; }
	}

	// Prettify lines
	for ( int x = 0; x < W; x++ )
	{
		for ( int y = 0; y < H; y++ )
		{
			if ( !tiles[x][y] ) continue;

			unsigned int mask = 0;
			if ( y > 0 && tiles[x][y-1] )	mask |= (1 << 0);
			if ( x < W-1 && tiles[x+1][y] )	mask |= (1 << 1);
			if ( y < H-1 && tiles[x][y+1] )	mask |= (1 << 2);
			if ( x > 0 && tiles[x-1][y] )	mask |= (1 << 3);
			tiles[x][y] = double_lines[mask];
		}
	}

	// Random letters
	srand(timeGetTime());
	for ( int i = 0; i < 50; i++ )
	{
		int x = 10+rand()%70, y=2+rand()%21;
		if ( !tiles[x][y] ) tiles[x][y] = 'a'+rand()%26;
	}

	// Place some lights (magick numbers all the way, yay!)
	struct { int x, y, ro, ri; float i, s, r, g, b; } lights[3] =
	{
		{ 15, 12, 10, 10, 0.0f, 10.0f, 1.0f, 0.0f, 0.0f },
		{ 40, 12, 20, 20, 1.0f, 20.0f, 1.0f, 1.0f, 0.0f },
		{ 55, 20, 15, 15, 2.0f,  5.0f, 1.0f, 0.5f, 0.0f }
	};

	const int N_lights = sizeof(lights)/sizeof(lights[0]);
	const float PI = 3.1415926f;
	const int frames_per_second = 25;
	const int frame_duration = 1000/frames_per_second;

	// Color buffer
	struct { float r, g, b; } light[W][H];

	while ( true )
	{
		// Clear colors
		for ( int x = 0; x < W; x++ )
		{
			for ( int y = 0; y < H; y++ )
			{
				// All cells are gray initially
				light[x][y].r = light[x][y].g = light[x][y].b = 0.05f;
			}
		}

		// Calculate colors
		for ( int i = 0; i < N_lights; i++ )
		{
			// Update light positions
			lights[i].i += 2.0f * PI / (frames_per_second * lights[i].s);
			float xc = lights[i].x + lights[i].ro * std::cos(lights[i].i);
			float yc = lights[i].y + lights[i].ro * std::sin(lights[i].i);

			// Add light
			float r = lights[i].ri;
			for ( int x = int(xc)-(r+1); x < int(xc)+(r+1); x++ )
			{
				if ( x < 0 || x >= W ) continue;
				for ( int y = int(yc)-(r-1); y < int(yc)+(r+1); y++ )
				{
					if ( y < 0 || y >= H ) continue;
					float f = (r - std::sqrt(std::pow(x-xc, 2) + std::pow(y-yc, 2))) / r;
					if ( f < 0 ) f = 0;

					light[x][y].r += lights[i].r * f;
					light[x][y].g += lights[i].g * f;
					light[x][y].b += lights[i].b * f;
				}
			}
		}

		terminal_clear();
		for ( int x = 0; x < W; x++ )
		{
			for ( int y = 0; y < H; y++ )
			{
				if ( !tiles[x][y] ) continue;

				color_t color = color_from_argb
				(
					255,
					255*light[x][y].r,
					255*light[x][y].g,
					255*light[x][y].b
				);

				terminal_color(color);
				terminal_put(x, y, tiles[x][y]);
			}
		}
		terminal_refresh();

		int key = terminal_get();

		if ( key == VK_ESCAPE || key == VK_CLOSE ) break;
	}

	terminal_options("input.timeout=infinite");
}
