/*
 * FontViewer.cpp
 *
 *  Created on: Jun 23, 2013
 *      Author: Cfyz
 */

// ...

#include "BearLibTerminal.h"
#include "Demos.hpp"
#include <string>
#include <sstream>
#include <cmath>

void BasicFontViewer()
{
	terminal_options("title=BearLibTerminal demo: fonts");

	struct font
	{
		enum en_type { Default, Bitmap, TrueType };

		en_type type;
		const char* name;
		int width, height;
		const char* codepage;
	}
	fonts[] =
	{
		{ font::Default, "default", 8, 16, "" },
		{ font::Bitmap, "media\\Nobbins.png", 9, 12, "437" },
		{ font::TrueType, "media\\consola.ttf", 0, 0, "" },
		{ font::TrueType, "media\\cour.ttf", 0, 0, "" },
		{ font::TrueType, "media\\UbuntuMono-R.ttf", 0, 0, "" }
	};

	const int N_fonts = sizeof(fonts)/sizeof(fonts[0]);

	const char* modes[] =
	{
		"monochrome",
		"normal",
		"lcd"
	};

	const int N_modes = sizeof(modes)/sizeof(modes[0]);

	const int sizes[] =
	{
		8, 9, 10, 11, 12, 13, 14, 15
	};

	const int N_sizes = sizeof(sizes)/sizeof(sizes[0]);

	auto construct_font_description = [&](const font& f, int size, const std::string& mode) -> std::string
	{
		std::stringstream s;
		if ( f.type == font::Default )
		{
			s << "font.name=default";
		}
		else if ( f.type == font::Bitmap )
		{
			s << "font.name=" << f.name << "; ";
			s << "font.size=" << f.width << "x" << f.height << "; ";
			s << "font.codepage=" << f.codepage;
		}
		else
		{
			s << "font.name=" << f.name << "; ";
			s << "font.size=" << size << "; ";
			s << "font.mode=" << mode;
		}
		return s.str();
	};

	struct range
	{
		const char* name;
		int start, end;
	}
	ranges[] =
	{
		{ "Basic Latin",		0x0020, 0x007F },
		{ "Latin-1 Supplement",	0x00A0, 0x00FF },
		{ "Latin Extended-A",	0x0100, 0x017F },
		{ "Latin Extended-B",	0x0180, 0x024F },
		{ "Greek and Coptic",	0x0370, 0x03FF },
		{ "Cyrillic",			0x0400, 0x04FF },
		{ "Box Drawing",		0x2500, 0x257F },
		{ "Block Elements",		0x2580, 0x259F },
	};

	const int N_ranges = sizeof(ranges)/sizeof(ranges[0]);

	const char* menus[] =
	{
		"Font face",
		"Font size",
		"Font mode",
		"Range"
	};

	const int N_menus = sizeof(menus)/sizeof(menus[0]);

	int current_range = 0;
	int current_font = 0;
	int current_size = 2;
	int current_mode = 1;
	int current_menu = 0;

	struct value
	{
		int& reference;
		int upper_limit;
	}
	values[N_menus] =
	{
		{ current_font, N_fonts },
		{ current_size, N_sizes },
		{ current_mode, N_modes },
		{ current_range, N_ranges }
	};

	auto setup_font = [&]()
	{
		font& f = fonts[current_font];
		terminal_options(construct_font_description(f, sizes[current_size], modes[current_mode]).c_str());
	};

	color_t menu_basic = 0xFFFFFFFF;
	color_t menu_selected = 0xFFFF8000;
	color_t value_text = 0xFFA0A0A0;
	color_t value_more = 0xFF008000;
	color_t value_no_more = 0xFF404040;
	color_t value_disabled = value_no_more;
	color_t range_numeric = value_more;
	color_t range_text = menu_basic;

	setup_font();

	do
	{
		terminal_clear();

		// Menu
		for ( int i = 0; i < N_menus; i++ )
		{
			terminal_color(i == current_menu? menu_selected: menu_basic);
			terminal_printf(1, 1+i, "%s", menus[i]);
		}

		font& f = fonts[current_font];
		range& r = ranges[current_range];
		int n = 0;

		// Values: font face
		terminal_printf
		(
			11, 1+0, "[color=%d]< [color=%d]%s [color=%d]>",
			current_font > 0? value_more: value_no_more,
			value_text,
			f.name,
			current_font < N_fonts-1? value_more: value_no_more
		);

		// Values: font size
		if ( f.type != font::TrueType )
		{
			terminal_printf(11, 1+1, "[color=%d]< %dx%d >", value_disabled, f.width, f.height);
		}
		else
		{
			terminal_printf
			(
				11, 1+1, "[color=%d]< [color=%d]%d [color=%d]>",
				current_size > 0? value_more: value_no_more,
				value_text,
				sizes[current_size],
				current_size < N_sizes-1? value_more: value_no_more
			);
		}

		// Values: font rasterization mode
		if ( f.type != font::TrueType )
		{
			terminal_printf(11, 1+2, "[color=%d]< n/a >", value_disabled);
		}
		else
		{
			terminal_printf
			(
				11, 1+2, "[color=%d]< [color=%d]%s [color=%d]>",
				current_mode > 0? value_more: value_no_more,
				value_text,
				modes[current_mode],
				current_mode < N_modes-1? value_more: value_no_more
			);
		}

		// Values: character range
		terminal_printf
		(
			11, 1+3, "[color=%d]< [color=%d]%s [color=%d]>",
			current_range > 0? value_more: value_no_more,
			value_text,
			r.name,
			current_range < N_ranges-1? value_more: value_no_more
		);

		terminal_color(0xFF808000);
		terminal_printf(1, 6, "Tip: use arrow keys");

		terminal_printf(1, 23, "[color=dark gray]%dx%d", terminal_state(VK_CELL_WIDTH), terminal_state(VK_CELL_HEIGHT));

		// Separator line
		terminal_color(value_disabled);
		for ( int i = 1; i < 24; i++ ) terminal_put(80-33-6, i, 0x2502);

		// Characters in the range
		terminal_color(range_numeric);
		for ( int column = 0; column < 16; column++ )
		{
			terminal_printf(80-33+1+column*2, 1, "%X", column);
		}

		int start = 16*std::floor(r.start/16.0);
		for ( int i = start; i < r.end; )
		{
			int row = (i-start)/16;
			terminal_color(range_numeric);
			terminal_printf(80-33-4, 2+row, "%04X", i);

			terminal_color(range_text);
			for ( int column = 0; column < 16; column++, i++ )
			{
				terminal_put(80-33+1+column*2, 2+row, i);
			}
		}

		terminal_refresh();

		int code = terminal_get();

		if ( code == VK_CLOSE || code == VK_ESCAPE )
		{
			break;
		}
		else if ( code == VK_UP )
		{
			if ( current_menu > 0 ) current_menu -= 1;
		}
		else if ( code == VK_DOWN )
		{
			if ( current_menu < N_menus-1 ) current_menu += 1;
		}
		else if ( code == VK_LEFT )
		{
			bool applicable = f.type != font::Bitmap || current_menu < 1 || current_menu > 2;
			bool affects = current_menu < 3;

			if ( applicable )
			{
				value& v = values[current_menu];
				if ( v.reference > 0 )
				{
					v.reference -= 1;
					if ( affects ) setup_font();
				}
			}
		}
		else if ( code == VK_RIGHT )
		{
			bool applicable = f.type != font::Bitmap || current_menu < 1 || current_menu > 2;
			bool affects = current_menu < 3;

			if ( applicable )
			{
				value& v = values[current_menu];
				if ( v.reference < v.upper_limit-1 )
				{
					v.reference += 1;
					if ( affects ) setup_font();
				}
			}
		}
	}
	while ( true );
}
