/*
 * Keyboard.cpp
 *
 *  Created on: Jun 23, 2013
 *      Author: Cfyz
 */

// ...

#include "BearLibTerminal.h"
#include "Demos.hpp"

void Keyboard()
{
	terminal_options("title=BearLibTerminal demo: basic keyboard input");
	terminal_options("input.events=keypress+keyrelease");
	terminal_blending(BLENDING_ADDITIVE);

	//
	//	"┌───┐┌──┬──┬──┬──┐┌──┬──┬──┬──┐┌──┬───┬───┬───┐┌─────┬─────┬───────┐",
	//	"│ESC││F1│F2│F3│F4││F5│F6│F7│F8││F9│F10│F11│F12││PRSCR│SCRLK│ PAUSE │",
	//	"└───┘└──┴──┴──┴──┘└──┴──┴──┴──┘└──┴───┴───┴───┘└─────┴─────┴───────┘",
	//	"┌───┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─────┐┌────┬────┬────┐┌───┬───┬───┬───┐",
	//	"│ ` │1│2│3│4│5│6│7│8│9│0│─│=│BCKSP││INS │HOME│PGUP││NUM│ / │ * │ ─ │",
	//	"├───┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬────┤├────┼────┼────┤├───┼───┼───┼───┤",
	//	"│TAB │q│w│e│r│t│y│u│i│o│p│[│]│  E ││DEL │END │PGDN││ 7 │ 8 │ 9 │   │",
	//	"├────┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┐ N │└────┴────┴────┘├───┼───┼───┤ + │",
	//	"│CAPS │a│s│d│f│g│h│j│k│l│;│'│\│ T │                │ 4 │ 5 │ 6 │   │",
	//	"├─────┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴─┴───┤     ┌────┐     ├───┼───┼───┼───┤",
	//	"│SHIFT │z│x│c│v│b│n│m│,│.│/│ SHIFT│     │ UP │     │ 1 │ 2 │ 3 │ E │",
	//	"├────┬─┴┬┴─┴┬┴─┴─┴┬┴─┴┬┴─┼─┴─┬────┤┌────┼────┼────┐├───┴───┼───┤ N │",
	//	"│CTRL│LW│ALT│SPACE│ALT│RW│CTX│CTRL││LEFT│DOWN│RGHT││ 0     │ . │ T │",
	//	"└────┴──┴───┴─────┴───┴──┴───┴────┘└────┴────┴────┘└───────┴───┴───┘"
	//

	const char* grid[] =
	{
		"┌───┐┌──┬──┬──┬──┐┌──┬──┬──┬──┐┌──┬───┬───┬───┐┌─────┬─────┬───────┐",
		"│   ││  │  │  │  ││  │  │  │  ││  │   │   │   ││     │     │       │",
		"└───┘└──┴──┴──┴──┘└──┴──┴──┴──┘└──┴───┴───┴───┘└─────┴─────┴───────┘",
		"┌───┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─────┐┌────┬────┬────┐┌───┬───┬───┬───┐",
		"│   │ │ │ │ │ │ │ │ │ │ │ │ │     ││    │    │    ││   │   │   │   │",
		"├───┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬────┤├────┼────┼────┤├───┼───┼───┼───┤",
		"│    │ │ │ │ │ │ │ │ │ │ │ │ │    ││    │    │    ││   │   │   │   │",
		"├────┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┐   │└────┴────┴────┘├───┼───┼───┤   │",
		"│     │ │ │ │ │ │ │ │ │ │ │ │ │   │                │   │   │   │   │",
		"├─────┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴┬┴─┴───┤     ┌────┐     ├───┼───┼───┼───┤",
		"│      │ │ │ │ │ │ │ │ │ │ │      │     │    │     │   │   │   │   │",
		"├────┬─┴┬┴─┴┬┴─┴─┴┬┴─┴┬┴─┼─┴─┬────┤┌────┼────┼────┐├───┴───┼───┤   │",
		"│    │  │   │     │   │  │   │    ││    │    │    ││       │   │   │",
		"└────┴──┴───┴─────┴───┴──┴───┴────┘└────┴────┴────┘└───────┴───┴───┘"
	};

	const size_t N_grid_lines = sizeof(grid)/sizeof(grid[0]);

	const auto fill_rect = [](int x, int y, int w, int h, color_t color)
	{
		terminal_color(color);

		for ( int i = x; i < x+w; i++ )
		{
			for ( int j = y; j < y+h; j++ )
			{
				terminal_put(i, j, 0x2588);
			}
		}

		for ( int i = x; i < x+w; i++ )
		{
			terminal_put(i, y-1, 0x2584);
			terminal_put(i, y+h, 0x2580);
		}

		for ( int j = y; j < y+h; j++ )
		{
			terminal_put(x-1, j, 0x2590);
			terminal_put(x+w, j, 0x258C);
		}

		terminal_put(x-1, y-1, 0x2597);
		terminal_put(x-1, y+h, 0x259D);
		terminal_put(x+w, y-1, 0x2596);
		terminal_put(x+w, y+h, 0x2598);
	};


	struct key_t
	{
		uint8_t vk;
		int x, y, w, h;
		const char* caption;
	};

	key_t keys[] =
	{
		// Functional keys row
		{ VK_ESCAPE, 1, 1, 3, 1, "ESC" },
		{ VK_F1, 6, 1, 2, 1, "F1" },
		{ VK_F2, 9, 1, 2, 1, "F2" },
		{ VK_F3, 12, 1, 2, 1, "F3" },
		{ VK_F4, 15, 1, 2, 1, "F4" },
		{ VK_F5, 19, 1, 2, 1, "F5" },
		{ VK_F6, 22, 1, 2, 1, "F6" },
		{ VK_F7, 25, 1, 2, 1, "F7" },
		{ VK_F8, 28, 1, 2, 1, "F8" },
		{ VK_F9, 32, 1, 2, 1, "F9" },
		{ VK_F10, 35, 1, 3, 1, "F10" },
		{ VK_F11, 39, 1, 3, 1, "F11" },
		{ VK_F12, 43, 1, 3, 1, "F12" },
		{ VK_PAUSE, 60, 1, 7, 1, " PAUSE " },
		// First row
		{ VK_GRAVE, 1, 4, 3, 1, " ` " },
		{ VK_1, 5, 4, 1, 1, "1" },
		{ VK_2, 7, 4, 1, 1, "2" },
		{ VK_3, 9, 4, 1, 1, "3" },
		{ VK_4, 11, 4, 1, 1, "4" },
		{ VK_5, 13, 4, 1, 1, "5" },
		{ VK_6, 15, 4, 1, 1, "6" },
		{ VK_7, 17, 4, 1, 1, "7" },
		{ VK_8, 19, 4, 1, 1, "8" },
		{ VK_9, 21, 4, 1, 1, "9" },
		{ VK_0, 23, 4, 1, 1, "0" },
		{ VK_MINUS, 25, 4, 1, 1, "-" },
		{ VK_EQUALS, 27, 4, 1, 1, "=" },
		{ VK_BACKSPACE, 29, 4, 5, 1, "BCKSP" },
		// Second row
		{ VK_TAB, 1, 6, 4, 1, "TAB" },
		{ VK_Q, 6, 6, 1, 1, "q" },
		{ VK_W, 8, 6, 1, 1, "w" },
		{ VK_E, 10, 6, 1, 1, "e" },
		{ VK_R, 12, 6, 1, 1, "r" },
		{ VK_T, 14, 6, 1, 1, "t" },
		{ VK_Y, 16, 6, 1, 1, "y" },
		{ VK_U, 18, 6, 1, 1, "u" },
		{ VK_I, 20, 6, 1, 1, "i" },
		{ VK_O, 22, 6, 1, 1, "o" },
		{ VK_P, 24, 6, 1, 1, "p" },
		{ VK_LBRACKET, 26, 6, 1, 1, "[[" },
		{ VK_RBRACKET, 28, 6, 1, 1, "]]" },
		// Third row
		// { VK_, 1, 8, 5, 1, "a" },
		{ VK_A, 7, 8, 1, 1, "a" },
		{ VK_S, 9, 8, 1, 1, "s" },
		{ VK_D, 11, 8, 1, 1, "d" },
		{ VK_F, 13, 8, 1, 1, "f" },
		{ VK_G, 15, 8, 1, 1, "g" },
		{ VK_H, 17, 8, 1, 1, "h" },
		{ VK_J, 19, 8, 1, 1, "j" },
		{ VK_K, 21, 8, 1, 1, "k" },
		{ VK_L, 23, 8, 1, 1, "l" },
		{ VK_SEMICOLON, 25, 8, 1, 1, ";" },
		{ VK_APOSTROPHE, 27, 8, 1, 1, "'" },
		{ VK_BACKSLASH, 29, 8, 1, 1, "\\" },
		// Fourth row
		{ VK_SHIFT, 1, 10, 6, 1, "SHIFT" },
		{ VK_Z, 8, 10, 1, 1, "z" },
		{ VK_X, 10, 10, 1, 1, "x" },
		{ VK_C, 12, 10, 1, 1, "c" },
		{ VK_V, 14, 10, 1, 1, "v" },
		{ VK_B, 16, 10, 1, 1, "b" },
		{ VK_N, 18, 10, 1, 1, "n" },
		{ VK_M, 20, 10, 1, 1, "m" },
		{ VK_COMMA, 22, 10, 1, 1, "," },
		{ VK_PERIOD, 24, 10, 1, 1, "." },
		{ VK_SLASH, 26, 10, 1, 1, "/" },
		{ VK_SHIFT, 28, 10, 6, 1, " SHIFT" },
		// Fifth row
		{ VK_CONTROL, 1, 12, 4, 1, "CTRL" },
		{ VK_SPACE, 13, 12, 5, 1, "SPACE" },
		{ VK_CONTROL, 30, 12, 4, 1, "CTRL" },
		// Navigation
		{ VK_INSERT, 36, 4, 4, 1, "INS" },
		{ VK_HOME, 41, 4, 4, 1, "HOME" },
		{ VK_PRIOR, 46, 4, 4, 1, "PGUP" },
		{ VK_DELETE, 36, 6, 4, 1, "DEL" },
		{ VK_END, 41, 6, 4, 1, "END" },
		{ VK_NEXT, 46, 6, 4, 1, "PGDN" },
		{ VK_UP, 41, 10, 4, 1, " UP " },
		{ VK_LEFT, 36, 12, 4, 1, "LEFT" },
		{ VK_DOWN, 41, 12, 4, 1, "DOWN" },
		{ VK_RIGHT, 46, 12, 4, 1, "RGHT" },
		// Numpad
		{ VK_DIVIDE, 56, 4, 3, 1, " / " },
		{ VK_MULTIPLY, 60, 4, 3, 1, " * " },
		{ VK_SUBTRACT, 64, 4, 3, 1, " - " },
		{ VK_NUMPAD7, 52, 6, 3, 1, " 7 " },
		{ VK_NUMPAD8, 56, 6, 3, 1, " 8 " },
		{ VK_NUMPAD9, 60, 6, 3, 1, " 9 " },
		{ VK_ADD, 64, 6, 3, 3, " + " },
		{ VK_NUMPAD4, 52, 8, 3, 1, " 4 " },
		{ VK_NUMPAD5, 56, 8, 3, 1, " 5 " },
		{ VK_NUMPAD6, 60, 8, 3, 1, " 6 " },
		{ VK_NUMPAD1, 52, 10, 3, 1, " 1 " },
		{ VK_NUMPAD2, 56, 10, 3, 1, " 2 " },
		{ VK_NUMPAD3, 60, 10, 3, 1, " 3 " },
		{ VK_NUMPAD0, 52, 12, 7, 1, " 0 " },
		{ VK_DECIMAL, 60, 12, 3, 1, " . " },
	};

	const size_t N_keys = sizeof(keys)/sizeof(keys[0]);

	key_t unavailable_keys[] =
	{
		{ 0, 1, 8, 5, 1, "CAPS " },
		{ 0, 6, 12, 2, 1, "LW" },
		{ 0, 9, 12, 3, 1, "ALT" },
		{ 0, 19, 12, 3, 1, "ALT" },
		{ 0, 23, 12, 2, 1, "RW" },
		{ 0, 26, 12, 3, 1, "CTX" },
		{ 0, 48, 1, 5, 1, "PRSCR" },
		{ 0, 54, 1, 5, 1, "SCRLK" },
		{ 0, 52, 4, 3, 1, "NUM" }
	};

	const color_t normal_text = 0xFFFFFFFF;
	const color_t pressed_key = 0xFFBB6000;
	const color_t pressed_key_text = 0xFF000000;
	const color_t available_key_text = 0xFFBBBBBB;
	const color_t unavailable_key_text = 0xFF404040;
	const color_t grid_color = 0xFF606060;
	const color_t note_text_color = 0xFF008000;

	bool proceed = true;
	while ( proceed )
	{
		terminal_clear();

		for ( size_t i = 0; i < N_keys; i++ )
		{
			if ( terminal_state(keys[i].vk) )
			{
				fill_rect(6+keys[i].x, 1+keys[i].y, keys[i].w, keys[i].h, pressed_key);
				terminal_color(pressed_key_text);
				terminal_printf(6+keys[i].x, 1+keys[i].y, "%s", keys[i].caption);
			}
			else
			{
				terminal_color(available_key_text);
				terminal_printf(6+keys[i].x, 1+keys[i].y, "%s", keys[i].caption);
			}
		}

		// Special case: Enter keys
		if ( terminal_state(VK_RETURN) )
		{
			// Main keyboard
			fill_rect(6+30, 1+6, 4, 1, pressed_key);
			fill_rect(6+31, 1+8, 3, 1, pressed_key);
			// Numpad
			fill_rect(6+64, 1+10, 3, 3, pressed_key);
		}

		terminal_color(terminal_state(VK_RETURN)? pressed_key_text: available_key_text);
		// Main keyboard
		terminal_put(6+32, 1+6+0, 'E');
		terminal_put(6+32, 1+6+1, 'N');
		terminal_put(6+32, 1+6+2, 'T');
		// Numpad
		terminal_put(6+65, 1+10+0, 'E');
		terminal_put(6+65, 1+10+1, 'N');
		terminal_put(6+65, 1+10+2, 'T');

		terminal_color(grid_color);
		for ( size_t i=0; i<N_grid_lines; i++ )
		{
			terminal_printf(6, 1+i, "%s", grid[i]);
		}

		terminal_color(unavailable_key_text);
		for ( auto& i: unavailable_keys )
		{
			terminal_printf(6+i.x, 1+i.y, "%s", i.caption);
		}

		int n = 6;
		terminal_color(note_text_color);
		n += terminal_printf(n, 1+15, "NOTE:");
		terminal_color(normal_text);
		n += terminal_printf(n, 1+15, " keys printed in dark gray color are not available by design.");

		n = 6;
		terminal_color(note_text_color);
		n += terminal_printf(n, 1+17, "NOTE:");
		terminal_color(normal_text);
		terminal_printf(n, 1+17, " for demonstration purposes sole Escape will not close this");

		n = 6;
		terminal_printf(n, 1+18, "demo, use Shift+Escape combination.");

		terminal_refresh();

		do
		{
			int key = terminal_get();
			if ( key == VK_CLOSE || (key == VK_ESCAPE && terminal_state(VK_SHIFT)) ) { proceed = false; break; }
		}
		while ( terminal_has_input() );
	}

	terminal_options("input.events=keypress");
}
