/*
 * Mouse.cpp
 *
 *  Created on: Jun 23, 2013
 *      Author: Cfyz
 */

// ...

#include "BearLibTerminal.h"
#include "Demos.hpp"

#include <sstream>

void Mouse()
{
	terminal_options("title=BearLibTerminal demo: mouse input");
	terminal_blending(BLENDING_ADDITIVE);

	bool proceed = true;

	bool keypress = true;
	bool keyrelease = true;
	bool mousemove = false;
	bool mousescroll = false;
	bool precise_mouse = false;
	int mlx = -1, mly = -1;
	int mrx = -1, mry = -1;

	const auto update_filter = [&]()
	{
		std::ostringstream s;
		s << "input.events=";
		if ( keypress ) s << "+keypress";
		if ( keyrelease ) s << "+keyrelease";
		if ( mousemove ) s << "+mousemove";
		if ( mousescroll ) s << "+mousescroll";
		terminal_options(s.str().c_str());
	};

	const auto update_precise = [&]()
	{
		std::string s = std::string("input.precise_mousemove=") + (precise_mouse? "true": "false");
		terminal_options(s.c_str());
	};

	while ( terminal_has_input() ) terminal_get();
	update_filter();

	int counter = 0;

	while ( proceed )
	{
		int mx = terminal_state(VK_MOUSE_X);
		int my = terminal_state(VK_MOUSE_Y);
		int mz = terminal_state(VK_MOUSE_WHEEL);
		int lmb = terminal_state(VK_LBUTTON);
		int rmb = terminal_state(VK_RBUTTON);

		terminal_clear();

		terminal_color(COLOR_WHITE);
		terminal_printf(1, 1, "Modify input events filter by setting/clearing following flags:");
		terminal_color(0xFF808080);
		terminal_printf(1, 2, "[[*]] Key press");
		terminal_color(COLOR_WHITE);
		terminal_printf(1, 3, "[[ ]] Key release");
		terminal_printf(1, 4, "[[ ]] Mouse movement");
		terminal_printf(1, 5, "[[ ]] Mouse scroll");

		terminal_printf(1, 7, "Additional input options:");
		terminal_printf(1, 8, "[[ ]] Precise mouse movement");

		terminal_color(COLOR_YELLOW);
		if ( keyrelease ) terminal_put(2, 3, '*');
		if ( mousemove ) terminal_put(2, 4, '*');
		if ( mousescroll ) terminal_put(2, 5, '*');
		if ( precise_mouse ) terminal_put(2, 8, '*');

		terminal_color(COLOR_WHITE);
		int n = terminal_printf(1, 10, "Received ");
		terminal_color(0xFFFF8000);
		n += terminal_printf(1+n, 10, "%d", counter);
		terminal_color(COLOR_WHITE);
		n += terminal_printf(1+n, 10, (counter == 1? " event": " events"));
		if ( counter >= 10000 )
		{
			terminal_color(0xFFFF8000);
			terminal_printf(1+n, 10, " 10k events! You are one persistent dude, LOL.");
		}

		terminal_color(COLOR_WHITE);
		terminal_printf(1, 12, "Current state: ");
		terminal_printf(1, 13, "LMB: ");
		terminal_printf(1, 14, "RMB: ");
		terminal_printf(1, 15, "Cursor: ");
		terminal_printf(1, 16, "Wheel: ");
		terminal_color(0xFFFF8000);
		terminal_printf(6, 13, "%s", (lmb? "Yes": "No"));
		terminal_printf(6, 14, "%s", (rmb? "Yes": "No"));
		terminal_printf(8, 16, "%d", mz);

		// Cursor positiondcrtf4l.;'kzdcrtf4l.;'kz
		n = 9;
		terminal_color(COLOR_WHITE);
		n += terminal_printf(n, 15, "([color=#FF8000]%d[/color], [color=#FF8000]%d[/color])", mx, my);
		if ( precise_mouse )
		{
			terminal_color(0xFF606060);
			terminal_printf(n, 15, " (%d, %d)", terminal_state(VK_MOUSE_PIXEL_X), terminal_state(VK_MOUSE_PIXEL_Y));
		}

		terminal_color(0xFF008000);
		terminal_printf(1, 18, "TIP: ");
		terminal_color(0xFF808080);
		terminal_printf(6, 18, "move and click mouse everywhere");

		terminal_color(0xFF008000);
		terminal_printf(1, 20, "NOTE: ");
		terminal_color(0xFF808080);
		terminal_printf(7, 20, "'current state' may not always show");
		terminal_printf(1, 21, "immediate values depending on filter flags");

		if ( mousemove )
		{
			terminal_color(0x80FFFFFF);
			for ( int x = 0; x < 80; x++ ) terminal_put(x, my, 0x2588);
			for ( int y = 0; y < 25; y++ ) if ( y != my ) terminal_put(mx, y, 0x2588);
		}

		terminal_color(0x8000FF00);
		terminal_put(mlx, mly, 0x2588);

		terminal_color(0x80FF00FF);
		terminal_put(mrx, mry, 0x2588);

		terminal_refresh();

		do
		{
			int code = terminal_get();
			counter += 1;

			if ( code == VK_ESCAPE || code == VK_CLOSE )
			{
				proceed = false;
				break;
			}
			else if ( code >= '2' && code <= '4' )
			{
				if ( code == '2' ) keyrelease = !keyrelease;
				if ( code == '3' ) mousemove = !mousemove;
				if ( code == '4' ) mousescroll = !mousescroll;
				update_filter();
			}
			else if ( code == VK_LBUTTON )
			{
				mx = terminal_state(VK_MOUSE_X);
				my = terminal_state(VK_MOUSE_Y);

				if ( mx >= 1 && mx <= 3 && ((my >= 3 && my <= 7) || my == 8) )
				{
					if ( my == 3 ) keyrelease = !keyrelease;
					if ( my == 4 ) mousemove = !mousemove;
					if ( my == 5 ) mousescroll = !mousescroll;
					if ( my == 8 ) precise_mouse = !precise_mouse;
					update_filter();
					update_precise();
				}
				else
				{
					mlx = mx;
					mly = my;
				}
			}
			else if ( code == VK_RBUTTON )
			{
				mrx = terminal_state(VK_MOUSE_X);
				mry = terminal_state(VK_MOUSE_Y);
			}
		}
		while ( terminal_has_input() );
	}

	terminal_options("input.events=keypress; input.precise_mousemove=false");
}
