/*
 * Speed.cpp
 *
 *  Created on: Jun 23, 2013
 *      Author: Cfyz
 */

// ...

#include "BearLibTerminal.h"
#include "Demos.hpp"

#include <Windows.h>

void Speed()
{
	terminal_options("title=BearLibTerminal demo: speed");
	terminal_blending(BLENDING_NONE);

	unsigned int shift_f = 0, shift_b = 0;
	struct color3_t { int r, g, b; };

	const auto get_shifted_color = [=](int shift) -> color3_t
	{
		float f = (shift%80) / 80.0f;
		color3_t c = { 0 };

		if ( f < 0.33f )
		{
			c.r = 255 * ((0.33f - f) / 0.33f);
			c.g = 255 * ((f - 0.0f) / 0.33f);
		}
		else if ( f < 0.66f )
		{
			c.g = 255 * ((0.66f - f) / 0.33f);
			c.b = 255 * ((f - 0.33f) / 0.33f);
		}
		else
		{
			c.b = 255 * ((1.0f - f) / 0.33f);
			c.r = 255 * ((f - 0.66f) / 0.33f);
		}

		if ( c.r > 255 ) c.r = 255;
		if ( c.g > 255 ) c.g = 255;
		if ( c.b > 255 ) c.b = 255;

		return c;
	};

	const auto get_highlighted_color = [=](color3_t c) -> color_t
	{
		c.r *= 2;
		c.g *= 2;
		c.b *= 2;

		if ( c.r > 255 ) c.r = 255;
		if ( c.g > 255 ) c.g = 255;
		if ( c.b > 255 ) c.b = 255;

		return 0xFF000000 | (c.r << 16) | (c.g << 8) | c.b;
	};

	const auto get_dimmed_color = [=](color3_t c) -> color_t
	{
		c.r /= 2;
		c.g /= 2;
		c.b /= 2;

		return 0xFF000000 | (c.r << 16) | (c.g << 8) | c.b;
	};

	color_t shifted_f[80], shifted_b[80];
	for ( int i = 0; i < 80; i++ )
	{
		color3_t c = get_shifted_color(i);
		shifted_b[i] = get_highlighted_color(c);
		shifted_f[i] = get_dimmed_color(c);
	}

	bool proceed = true;

	uint64_t fps_update_time = timeGetTime();
	int fps_counter = 0;
	int fps_value = 0;

	while ( proceed )
	{
		terminal_clear();
		for ( int y = 0; y < 25; y++ )
		{
			for ( int x = 0; x < 80; x++ )
			{
				terminal_bkcolor(shifted_b[(shift_b+x+y)%80]);
				terminal_color(shifted_f[(shift_f+x-y)%80]);
				terminal_put(x, y, 'A' + (x + y)%26);
			}
		}
		terminal_bkcolor(0);
		terminal_color(COLOR_WHITE);
		terminal_printf(0, 24, "FPS = %d", fps_value);
		terminal_refresh();

		fps_counter += 1;
		uint64_t time = timeGetTime();
		if ( time > fps_update_time + 1000 )
		{
			fps_value = fps_counter;
			fps_counter = 0;
			fps_update_time = time;
		}

		while ( terminal_has_input() )
		{
			int code = terminal_get();
			if ( code == VK_ESCAPE || code == VK_CLOSE )
			{
				proceed = false;
				break;
			}
		}

		shift_f -= 1;
		shift_b += 1;
	}
}
