/*
 * Sprites.cpp
 *
 *  Created on: Jun 24, 2013
 *      Author: Cfyz
 */

#include "BearLibTerminal.h"
#include "Demos.hpp"

#include <vector>
#include <utility>

std::vector<std::pair<std::string, std::string>> sprites =
{
	{"Hell Knight", "tile_01.png"},
	{"Rusty golem", "tile_02.png"},
	{"Arachne", "tile_03.png"},
	{"Pure demon", "tile_04.png"},
	{"Succubus", "tile_05.png"},
	{"Green dragon", "tile_06.png"},
	{"Golden dragon", "tile_07.png"},
	{"Red dragon", "tile_08.png"},
	{"Black dragon", "tile_09.png"},
	{"Divine dragon", "tile_10.png"}
};

std::string SpriteFilename(const std::string& name)
{
	return std::string("media\\sprites\\")+name;
}

void LoadSprite(int index)
{
	terminal_use_image(0xE100, SpriteFilename(sprites[index].second).c_str());
}

void UnloadSprite()
{
	terminal_use_image(0xE100, NULL);
}

void Sprites()
{
	terminal_options("title=BearLibTerminal demo: sprites");
	terminal_options("font.name=default");

	terminal_blending(BLENDING_ADDITIVE);
	terminal_color(COLOR_WHITE);
	
	int index = 0;
	LoadSprite(index);
	
	const wchar_t
		kLowerRightQuad = 0x2597,	kLowerHalf = 0x2584,	kLowerLeftQuad = 0x2596,
		kRightHalf = 0x2590,		kFullBlock = 0x2588,	kLeftHalf = 0x258C,
		kUpperRightQuad = 0x259D,	kUpperHalf = 0x2580,	kUpperLeftQuad = 0x2598;

	const wchar_t
		kUpperLeftCorner = 0x250C,	kHorisontal = 0x2500,	kUpperRightCorner = 0x2510,
		kVertical = 0x2502,			/* none */				/* kVertical */
		kLowerLeftCorner = 0x2514,	/* kHorisontal */		kLowerRightCorner = 0x2518;

	color_t back = color_from_argb(255, 71, 108, 108);
	color_t fore = color_from_name("white");

	while (true)
	{
		terminal_clear();
		
		terminal_printf(5+12, 7-2, "[color=#008000]TIP:[/color] [color=#808080]use up/down arrow keys");
		
		terminal_color(fore);
		for (int i=0; i<sprites.size(); i++)
		{
			terminal_printf(5+12, 7+i, "%s", sprites[i].first.c_str());
			if (i == index) terminal_put(5+12-2, 7+i, '>');
		}
		
		// Put the image on screen
		terminal_put(40, 10, 0xE100); // sprite is 8x4 cells
		
		// Put a frame around the image (not really necessary but nice)
		for (int i=0; i<8; i++)
		{
			terminal_color(back);
			terminal_put(40+i, 10-1, kLowerHalf);
			terminal_put(40+i, 10+4, kUpperHalf);
			terminal_color(fore);
			terminal_put(40+i, 10-1, kHorisontal);
			terminal_put(40+i, 10+4, kHorisontal);
		}
		for (int i=0; i<4; i++)
		{
			terminal_color(back);
			terminal_put(40-1, 10+i, kRightHalf);
			terminal_put(40+8, 10+i, kLeftHalf);
			terminal_color(fore);
			terminal_put(40-1, 10+i, kVertical);
			terminal_put(40+8, 10+i, kVertical);
		}
		terminal_color(back);
		terminal_put(40-1, 10-1, kLowerRightQuad);
		terminal_put(40-1, 10+4, kUpperRightQuad);
		terminal_put(40+8, 10+4, kUpperLeftQuad);
		terminal_put(40+8, 10-1, kLowerLeftQuad);
		terminal_color(fore);
		terminal_put(40-1, 10-1, kUpperLeftCorner);
		terminal_put(40-1, 10+4, kLowerLeftCorner);
		terminal_put(40+8, 10+4, kLowerRightCorner);
		terminal_put(40+8, 10-1, kUpperRightCorner);
		
		
		terminal_refresh();

		int key = terminal_get();
		if (key == VK_ESCAPE || key == VK_CLOSE)
		{
			break;
		}
		else if (key == VK_UP && index > 0)
		{
			index -= 1;
			LoadSprite(index);
		}
		else if (key == VK_DOWN && index < sprites.size()-1)
		{
			index += 1;
			LoadSprite(index);
		}
	}
	
	UnloadSprite();
}